/* Copyright (C) 1999-2000 Hans Petter K. Jansson
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * You can contact the library's author by sending e-mail to <hpj@styx.net>.
 */

#ifndef BT_H
#define BT_H 1

#ifdef LIBFLUX_BUILD
# include "types.h"
# include "bmem.h"
#else
# include <flux/types.h>
# include <flux/bmem.h>
#endif


#ifndef _BT_DEFINED
typedef struct _bt_node
{
  u32 is_red : 1;
  void *key;
  void *data;
  struct _bt_node *left, *right;
}
BTN;


typedef struct _bt
{
  BMEM *bmem;
  BTN head, z;
}
BT;
#endif


BT *bt_new();
BT *bt_new_without_bmem();
void bt_del(BT *bt);

void bt_insert(BT *bt, void *key, void *data);
void bt_move(BT *btd, BT *bts, void *key);
void bt_remove(BT *bt, void *key);

#define bt_is_empty(bt) ((bt)->head.right == &((bt)->z))
#define bt_get_root(bt) ((bt)->head.right->data)
void *bt_find(BT *bt, void *key);
void *bt_find_first_in_range(BT *bt, void *min, void *max);
int bt_for_all(BT *bt, int (*func)(void *, void *));


#endif /* BT_H */
